@@ -1,5 +1,5 @@
+import json
 import logging
-import os
 import re
 import subprocess
 from functools import wraps
@@ -14,7 +14,7 @@ from rclone_python.remote_types import RemoteTypes
 def __check_installed(func):
     @wraps(func)
     def wrapper(*args, **kwargs):
-        if not is_installed:
+        if not is_installed():
             raise Exception('rclone is not installed on this system. Please install it here: https://rclone.org/')
 
         return func(*args, **kwargs)
@@ -23,11 +23,19 @@ def __check_installed(func):
 
 
 def is_installed() -> bool:
+    """
+    :return: True if rclone is correctly installed on the system.
+    """
     return which('rclone') is not None
 
 
 @__check_installed
 def check_remote_existing(remote_name: str) -> bool:
+    """
+    Returns True, if the specified rclone remote is already configured.
+    :param remote_name: The name of the remote to check.
+    :return: True if the remote exists, False otherwise.
+    """
     # get the available remotes
     remotes = get_remotes()
 
@@ -63,16 +71,103 @@ def create_remote(remote_name, remote_type: Union[str, RemoteTypes], client_id=N
         raise Exception(f'A rclone remote with the name \'{remote_name}\' already exists!')
 
 
-def copy(in_path: str, out_path: str, ignore_existing=False, listener: Callable[[Dict], None] = None):
-    _copy_move(in_path, out_path, ignore_existing=ignore_existing, move_files=False, listener=listener)
+def copy(in_path: str, out_path: str, ignore_existing=False, show_progress=True,
+         listener: Callable[[Dict], None] = None):
+    """
+    Copies a file or a directory from a src path to a destination path.
+    :param in_path: The source path to use. Specify the remote with 'remote_name:path_on_remote'
+    :param out_path: The destination path to use. Specify the remote with 'remote_name:path_on_remote'
+    :param ignore_existing: If True, all existing files are ignored and not overwritten.
+    :param show_progress: If true, show a progressbar.
+    :param listener: An event-listener that is called with every update of rclone.
+    """
+    _copy_move(in_path, out_path, ignore_existing=ignore_existing, move_files=False, show_progress=show_progress,
+               listener=listener)
 
 
-def move(in_path: str, out_path: str, ignore_existing=False, listener: Callable[[Dict], None] = None):
-    _copy_move(in_path, out_path, ignore_existing=ignore_existing, move_files=True, listener=listener)
+def move(in_path: str, out_path: str, ignore_existing=False, show_progress=True,
+         listener: Callable[[Dict], None] = None):
+    """
+    Moves a file or a directory from a src path to a destination path.
+    :param in_path: The source path to use. Specify the remote with 'remote_name:path_on_remote'
+    :param out_path: The destination path to use. Specify the remote with 'remote_name:path_on_remote'
+    :param ignore_existing: If True, all existing files are ignored and not overwritten.
+    :param show_progress: If true, show a progressbar.
+    :param listener: An event-listener that is called with every update of rclone.
+    """
+    _copy_move(in_path, out_path, ignore_existing=ignore_existing, move_files=True, show_progress=show_progress,
+               listener=listener)
 
 
 @__check_installed
-def _copy_move(in_path: str, out_path: str, ignore_existing=False, move_files=False,
+def get_remotes() -> List[str]:
+    """
+    :return: A list of all available remotes.
+    """
+    remotes = subprocess.check_output('rclone listremotes', shell=True, encoding='UTF-8').split()
+    if remotes is None:
+        remotes = []
+
+    print(remotes)
+
+    return remotes
+
+
+@__check_installed
+def purge(path: str):
+    """
+    Purges the specified folder. This means that unlike with delete, also all the folders are removed.
+    :param path: The path of the folder that should be purged.
+    """
+    process = subprocess.run(f'rclone purge {path}', stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                             shell=True)
+    if process.returncode == 0:
+        logging.info(f'Successfully purged {path}')
+    else:
+        raise Exception(
+            f'Purging path \"{path}\" failed with error message:\n{process.stderr}')
+
+
+@__check_installed
+def delete(path: str):
+    """
+    Deletes a file or a folder. When deleting a folder, all the files in it and it's subdirectories are removed,
+    but not the folder structure itself.
+    :param path: The path of the folder that should be deleted.
+    """
+    command = 'delete'
+    process = subprocess.run(f'rclone {command} {path}', stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                             shell=True)
+    if process.returncode == 0:
+        logging.info(f'Successfully deleted {path}')
+    else:
+        raise Exception(
+            f'Deleting path \"{path}\" failed with error message:\n{process.stderr}')
+
+
+@__check_installed
+def ls(path: str, max_depth: Union[int, None] = None) -> List[Dict[str, Union[int, str]]]:
+    """
+    Lists the files in a directory.
+    :param path: The path to the folder that should be examined.
+    :param max_depth:
+    :return: List of dicts containing file properties.
+    """
+    command = f'rclone lsjson {path}'
+
+    if max_depth:
+        command += f" --max-depth {max_depth}"
+
+    process = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, encoding='utf-8')
+
+    if process.returncode == 0:
+        return json.loads(process.stdout)
+    else:
+        raise Exception(f'ls operation on {path} failed with {process.stderr}')
+
+
+@__check_installed
+def _copy_move(in_path: str, out_path: str, ignore_existing=False, move_files=False, show_progress=True,
                listener: Callable[[Dict], None] = None):
     if move_files:
         command = f'rclone move'
@@ -92,9 +187,9 @@ def _copy_move(in_path: str, out_path: str, ignore_existing=False, move_files=Fa
     command += f' {out_path}'
 
     # execute the upload command
-    process = _rclone_progress(command, prog_title, listener=listener)
+    process = _rclone_progress(command, prog_title, listener=listener, show_progress=show_progress)
 
-    if process.wait() == os.EX_OK:
+    if process.wait() == 0:
         logging.info('Cloud upload completed.')
     else:
         _, err = process.communicate()
@@ -102,74 +197,46 @@ def _copy_move(in_path: str, out_path: str, ignore_existing=False, move_files=Fa
                         f' failed with error message:\n{err.decode("utf-8")}')
 
 
-@__check_installed
-def get_remotes() -> List[str]:
-    remotes = subprocess.check_output('rclone listremotes', shell=True, encoding='UTF-8').split()
-    if remotes is None:
-        remotes = []
-
-    print(remotes)
-
-    return remotes
-
-
-@__check_installed
-def purge(path: str):
-    process = subprocess.run(f'rclone purge {path}', stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                             shell=True)
-    if process.returncode == os.EX_OK:
-        logging.info(f'Successfully purged {path}')
-    else:
-        raise Exception(
-            f'Purging path \"{path}\" failed with error message:\n{process.stderr}')
-
-
-@__check_installed
-def delete(path: str):
-    command = 'delete'
-    process = subprocess.run(f'rclone {command} {path}', stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                             shell=True)
-    if process.returncode == os.EX_OK:
-        logging.info(f'Successfully deleted {path}')
-    else:
-        raise Exception(
-            f'Deleting path \"{path}\" failed with error message:\n{process.stderr}')
-
-
-def _rclone_progress(command: str, pbar_title: str, stderr=subprocess.PIPE,
+def _rclone_progress(command: str, pbar_title: str, stderr=subprocess.PIPE, show_progress=True,
                      listener: Callable[[Dict], None] = None) -> subprocess.Popen:
     process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=stderr, shell=True)
 
     buffer = ""
+    pbar = None
 
-    with tqdm(bar_format='{l_bar}{bar}| {n:.1f}/{total_fmt} {unit}{postfix}') as pbar:
-        pbar.set_description(pbar_title)
+    if show_progress:
+        pbar = tqdm(bar_format='{l_bar}{bar}| {n:.1f}/{total_fmt} {unit}{postfix}')
+        pbar.set_description(f'☁ {pbar_title}')
 
-        for c in iter(lambda: process.stdout.read(1), b''):
-            var = c.decode('utf-8')
-            if '\n' not in var:
-                buffer += var
-            else:
-                valid, update_dict = _extract_rclone_progress(buffer)
+    for c in iter(lambda: process.stdout.read(1), b''):
+        var = c.decode('utf-8')
+        if '\n' not in var:
+            buffer += var
+        else:
+            valid, update_dict = _extract_rclone_progress(buffer)
 
-                if valid:
+            if valid:
+                if show_progress:
                     pbar.set_postfix_str(f'{update_dict["transfer_speed"]:.1f} {update_dict["transfer_speed_unit"]}, '
                                          f'ETA: {update_dict["eta"]}')
                     pbar.total = update_dict['total_bits']
                     pbar.unit = update_dict['unit_total']
                     pbar.update(update_dict['sent_bits'] - pbar.n, )
 
-                    # call the listener
-                    if listener:
-                        listener(update_dict)
+                # call the listener
+                if listener:
+                    listener(update_dict)
 
-                    # reset the buffer
-                    buffer = ""
+                # reset the buffer
+                buffer = ""
 
+    if show_progress:
         if not pbar.total:
             # if no data is downloaded/ upload because the data is already present: manually set progress to 100%
             pbar.total = 1
             pbar.update()
+        pbar.close()
+
     return process
 
 
